# -*- coding: utf-8 -*-
"""
Biblium GUI Package
===================
Modern Tkinter-based desktop application for bibliometric analysis.

@author: Lan.Umek
@version: 2.5.0
"""

from biblium.gui.app import BibliumApp, main

__all__ = ["BibliumApp", "main"]
